/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIAQUICK_VERSION_H
#define QT_QTMULTIMEDIAQUICK_VERSION_H

#define QTMULTIMEDIAQUICK_VERSION_STR "6.11.1"

#define QTMULTIMEDIAQUICK_VERSION 0x0601101

#endif // QT_QTMULTIMEDIAQUICK_VERSION_H
